@@ -1,9 +1,9 @@
 prop.odds.subdist<-function(formula,data=sys.parent(),cause=1,beta=NULL,
 Nit=10,detail=0,start.time=0,max.time=NULL,id=NULL,n.sim=500,weighted.test=0,
-profile=1,sym=0,cens.model="KM",clusters=NULL,max.clust=1000,baselinevar=1,weights=NULL)
+profile=1,sym=0,cens.model="KM",clusters=NULL,max.clust=1000,baselinevar=1,weights=NULL,
+cens.weights=NULL)
 {
 ## {{{ 
-
 ## {{{ 
  if (!missing(cause)){
     if (length(cause)!=1) stop("Argument cause specifies the cause of interest, see help(prop.odds.subdist) for details.")
@@ -18,6 +18,7 @@ profile=1,sym=0,cens.model="KM",clusters=NULL,max.clust=1000,baselinevar=1,weigh
     residuals<-0;  robust<-1; resample.iid <- 1 
     m$cens.model <- m$cause <- m$sym<-m$profile <- m$max.time<- m$start.time<- m$weighted.test<- m$n.sim<-
     m$id<-m$Nit<-m$detail<-m$beta <- m$baselinevar <- m$clusters <- m$max.clust <- m$weights <-  NULL
+    m$cens.weights <- NULL
 
     special <- c("cluster")
     if (missing(data)) {
@@ -135,7 +136,7 @@ if ((!is.null(max.clust))) if (max.clust<antclust) {
 
 
 if ((is.null(beta)==FALSE)) {
-	if (length(beta)!=pg) beta <- rep(beta[1],pg); 
+	if (length(c(beta))!=pg) beta <- rep(beta[1],pg); 
 } else {
       beta<-coxph(Surv(eventtime,event)~desX)$coef
      if (max(abs(beta))>5) {
@@ -165,6 +166,7 @@ loglike<-0;
 
 ## {{{ censoring and estimator
 
+if (is.null(cens.weights)) { ## {{{ censoring model stuff with possible truncation
 if (cens.model=="KM") { ## {{{
     ud.cens<-survfit(Surv(time2,delta==cens.code)~+1);
     Gfit<-cbind(ud.cens$time,ud.cens$surv)
@@ -196,6 +198,13 @@ if (cens.model=="KM") { ## {{{
     KMti <- rep(1,length(time2)); KMtimes <- rep(1,length(times));
     }
     else  { stop('Unknown censoring model') }
+    }  else {
+       if (length(cens.weights)!=nx) stop("censoring weights must have length equal to nrow in data\n");  
+       KMti <- cens.weights
+       Gctimes <- rep(1,length(times)); 
+       ord2 <- order(time2) 
+       KMtimes<-Cpred(cbind(time2[ord2],cens.weights[ord2]),times)[,2]; 
+    }
 ## }}}
 
     if (is.null(weights)) weights <- rep(1,nx); 
@@ -225,7 +234,8 @@ nparout<- .C("posubdist2",
 	as.double(Rvcu),as.double(RVarbeta),as.double(test),
 	as.double(testOBS),as.double(Ut),as.double(simUt),
 	as.double(Uit),as.integer(id),as.integer(status),
-	as.integer(weighted.test),as.integer(rate.sim),as.double(score), as.double(cumAi),as.double(cumAiiid),as.integer(residuals), 
+	as.integer(weighted.test),as.integer(rate.sim),as.double(score), 
+	as.double(cumAi),as.double(cumAiiid),as.integer(residuals), 
 	as.double(loglike),as.integer(profile),as.integer(sym),
 	as.double(KMtimes),as.double(KMti),as.double(time2),
 	as.integer(causeS), as.integer(index-1), as.integer(baselinevar),
@@ -326,6 +336,7 @@ if (cens.model!="po") attr(ud,"type") <- "comprisk" else attr(ud,"type") <- "sur
 class(ud)<-"cox.aalen"
 return(ud); 
 } ## }}} 
+## }}} 
 
 predictpropodds <- function(out,X=NULL,times=NULL)
 {  ## {{{ 
@@ -343,3 +354,64 @@ pred <- HRR/(1+HRR)
 return(list(pred=pred,time=btimes))
 }  ## }}}
 
+prop.odds.subdist.ipw <- function(compriskformula,glmformula,data=sys.parent(),cause=1,
+			 max.clust=NULL,ipw.se=FALSE,...)
+{ ## {{{ 
+  ggl <- glm(glmformula,family='binomial',data=data)
+  mat <-  model.matrix(glmformula,data=data);
+  glmcovs <- attr(ggl$terms,"term.labels")
+  data$ppp <- predict(ggl,type='response')
+
+  dcc <- data[ggl$y==1,]
+  ppp <- dcc$ppp
+  udca <- prop.odds.subdist(compriskformula,data=dcc,cause=cause,weights=1/ppp,n.sim=0,
+		    max.clust=max.clust,...)  
+  ### iid of beta for comprisk model 
+  compriskiid <- udca$gamma.iid
+
+if (ipw.se==TRUE)  { ## {{{ 
+###requireNamespace("lava"); 
+###requireNamespace("NumDeriv"); 
+	glmiid <-   lava::iid(ggl)
+	mat <- mat[ggl$y==1,]
+	par <- coef(ggl)
+
+	compriskalpha <- function(par)
+	{ ## {{{ 
+	  rr <- mat %*% par
+	  pw <- c(exp(rr)/(1+exp(rr)))
+	  assign("pw",pw,envir=environment(compriskformula))
+	  ud <- prop.odds.subdist(compriskformula,data=dcc,
+			  cause=cause,
+			  weights=1/pw,baselinevar=0,beta=udca$gamma,
+			  Nit=1,n.sim=0,...)  
+	  ud$score
+	} ## }}} 
+
+	DU <-  numDeriv::jacobian(compriskalpha,par)
+	IDU <-  udca$D2linv %*% DU 
+	alphaiid <-t( IDU %*% t(glmiid))
+	###
+	iidfull <- alphaiid
+	###
+	iidfull[ggl$y==1,] <- compriskiid + alphaiid[ggl$y==1,]
+	###
+	var2 <- t(iidfull) %*% iidfull
+	se <- cbind(diag(var2)^.5); colnames(se) <- "se"
+} else { iidfull <- NULL; var2 <- NULL; se <- NULL} ## }}} 
+
+var.naive=udca$robvar.gamma
+se.naive=matrix(diag(var.naive)^.5,nrow(var.naive),1); 
+colnames(se.naive) <- "se.naive"
+
+res <- list(iid=iidfull,coef=udca$gamma,var.naive=var.naive,
+	    se.naive=se.naive,var=var2,se=se,
+	    comprisk.ipw=udca)
+class(res) <- "comprisk.ipw"
+return(res)
+} ## }}} 
+
+
+
+
+
